# Create a simulator object
set ns [new Simulator]

# Set trace file name
set traceallfile monarch.nam

# Set TCP trace file name
set trace_var [open monarch_tcp.state w]

# Remove old trace files
exec rm -f $traceallfile
exec rm -f $trace_var

# Open the NAM trace file and tell ns to put traces in the file
set allchan [open $traceallfile w]
$ns namtrace-all $allchan

# Use the Realtime scheduler; This means the ns clock will run in real time
$ns use-scheduler RealTime

# Set up three agents; The converter will sit in between src and dst
set src [new Agent/TCP]
set dst [new Agent/TCPSink]
set conv [new Agent/Converter]

# Set up three nodes, one for each agent
set node0 [$ns node]
set node1 [$ns node]
set node2 [$ns node]

# Set up links joining all nodes in both directions; We use only one direction links between any two nodes though
$ns simplex-link $node0 $node1 99Mb 1ms DropTail
$ns simplex-link $node1 $node2 99Mb 1ms DropTail
$ns simplex-link $node2 $node0 99Mb 1ms DropTail
$ns simplex-link $node1 $node0 99Mb 1ms DropTail
$ns simplex-link $node2 $node1 99Mb 1ms DropTail
$ns simplex-link $node0 $node2 99Mb 1ms DropTail

# Attach agents to respective nodes
$ns attach-agent $node0 $src
$ns attach-agent $node1 $conv
$ns attach-agent $node2 $dst

# Pass paramaters to the converter in the format:
# <Sender:SenderPort> <VirtualReceiver:VirtualReceiverPort> <VirtualSender:VirtualSenderPort> <Receiver:ReceiverPort>
$conv params 10.0.0.1:5001 10.0.0.2:5002 10.0.0.2:5003 10.0.0.1:5004

# Explicitly specify the path to be followed for packet transfer
# For e.g., the first command means that the target for packets leaving $src will be $conv
# and so on.
# This ensures that $conv (a) receives all the packets and (b) can forward all the packets
# We send the ns-acks directly from $dst to $src
$ns simplex-connect $src $conv
$ns simplex-connect $conv $dst
$ns simplex-connect $dst $src

if {$argc == 2} {
	$src set packetSize_ [lindex $argv 1]
} else {
	$src set packetSize_ 1460
}
# Calculate as PMTU - 40 B (TCP/IP headers)
# $src set packetSize_ 1460

$src set maxcwnd_ 3

# Create FTP application and attach it to the tcp agent
set ftp1 [new Application/FTP]
$ftp1 attach-agent $src

# Print all on a single trace event
$src set trace_all_ 1
# Print all variables in one line, update is everytime a variable changes
$src set trace_all_oneline_ 1

# Attach the trace file to the TCP Source
$src attach $trace_var

# Trace these variables
$src trace cwnd_
$src trace ssthresh_
$src trace ack_
$src trace dupacks_
$src trace rtt_
$src trace srtt_
$src trace maxseq_
$src trace highest_ack_
$src trace seqno_
$src trace rttvar_
$src trace backoff_
$src trace tcp_tick_

# We call this procedure in the end to generate the trace
proc finish {} {
    global ns allchan
    $ns flush-trace
    close $allchan
#    exec ../nam-1.11/nam em.nam &
    exit 0
}

# The file size is taken from command line
# If an argument is given, send a file of that size
# In the other case, send packets till 10 seconds
if {$argc > 0} {
	set fSize [lindex $argv 0]
	$ns at 0.1 "$ftp1 send $fSize"
} else {	
	$ns at 0.1 "$ftp1 start"
	$ns at 10.0 "$ftp1 stop"
}

# Finish the simulation when TCP is done with the transfer
$src proc done {} {"finish"}

$ns run
